@@ -1,4 +1,4 @@
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 
 use cargo_toml::Product;
 use log::debug;
@@ -46,8 +46,8 @@ impl BinFile {
         let dest_file_path = ctx.render("{ bin }-v{ version }{ binary-ext }")?;
         let dest = data.install_path.join(dest_file_path);
 
-        // Link at install dir + base name
-        let link = data.install_path.join(&base_name);
+        // Link at install dir + base-name{.extension}
+        let link = data.install_path.join(&ctx.render("{ bin }{ binary-ext }")?);
 
         Ok(Self {
             base_name,
@@ -70,8 +70,8 @@ impl BinFile {
         format!(
             "{} ({} -> {})",
             self.base_name,
-            self.dest.display(),
-            self.link.display()
+            self.link.display(),
+            self.link_dest().display()
         )
     }
 
@@ -102,18 +102,30 @@ impl BinFile {
             std::fs::remove_file(&self.link)?;
         }
 
+        let dest = self.link_dest();
         debug!(
             "Create link '{}' pointing to '{}'",
             self.link.display(),
-            self.dest.display()
+            dest.display()
         );
         #[cfg(target_family = "unix")]
-        std::os::unix::fs::symlink(&self.dest, &self.link)?;
+        std::os::unix::fs::symlink(dest, &self.link)?;
         #[cfg(target_family = "windows")]
-        std::os::windows::fs::symlink_file(&self.dest, &self.link)?;
+        std::os::windows::fs::symlink_file(dest, &self.link)?;
 
         Ok(())
     }
+
+    fn link_dest(&self) -> &Path {
+        #[cfg(target_family = "unix")]
+        {
+            Path::new(self.dest.file_name().unwrap())
+        }
+        #[cfg(target_family = "windows")]
+        {
+            &self.dest
+        }
+    }
 }
 
 /// Data required to get bin paths
