@@ -1,11 +1,11 @@
-use std::path::PathBuf;
+use std::{path::PathBuf, str::FromStr};
 
+use cargo_toml::{Package, Product};
 use log::{debug, error, info, warn, LevelFilter};
 use simplelog::{ColorChoice, ConfigBuilder, TermLogger, TerminalMode};
-
 use structopt::StructOpt;
-
 use tempdir::TempDir;
+use tokio::process::Command;
 
 use cargo_binstall::{
     bins,
@@ -20,7 +20,8 @@ struct Options {
     #[structopt()]
     name: String,
 
-    /// Filter for package version to install
+    /// Filter for package version to install, in Cargo.toml format.
+    /// Use `=1.2.3` to install a specific version.
     #[structopt(long, default_value = "*")]
     version: String,
 
@@ -104,10 +105,24 @@ async fn main() -> Result<(), anyhow::Error> {
     let manifest = load_manifest_path(manifest_path.join("Cargo.toml"))?;
     let package = manifest.package.unwrap();
 
+    let is_plain_version = semver::Version::from_str(&opts.version).is_ok();
+    if is_plain_version && package.version != opts.version {
+        warn!(
+            "You specified `--version {o}` but the package resolved that to '{p}', use `={o}` if you want an exact match",
+            o=opts.version, p=package.version
+        );
+
+        if !opts.no_confirm && !opts.dry_run && !confirm()? {
+            warn!("Installation cancelled");
+            return Ok(());
+        }
+    }
+
     let (mut meta, binaries) = (
         package
             .metadata
-            .map(|m| m.binstall)
+            .as_ref()
+            .map(|m| m.binstall.clone())
             .flatten()
             .unwrap_or(PkgMeta::default()),
         manifest.bin,
@@ -143,14 +158,37 @@ async fn main() -> Result<(), anyhow::Error> {
 
     // Try github releases, then quickinstall
     let mut fetchers = MultiFetcher::default();
-    fetchers.add(GhCrateMeta::new(&fetcher_data).await?);
-    fetchers.add(QuickInstall::new(&fetcher_data).await?);
+    fetchers.add(GhCrateMeta::new(&fetcher_data).await);
+    fetchers.add(QuickInstall::new(&fetcher_data).await);
 
-    let fetcher = fetchers.first_available().await.ok_or_else(|| {
-        error!("File does not exist remotely, cannot proceed");
-        anyhow::anyhow!("No viable remote package found")
-    })?;
+    match fetchers.first_available().await {
+        Some(fetcher) => {
+            install_from_package(
+                binaries,
+                fetcher,
+                install_path,
+                meta,
+                opts,
+                package,
+                pkg_path,
+                temp_dir,
+            )
+            .await
+        }
+        None => install_from_source(opts, package).await,
+    }
+}
 
+async fn install_from_package(
+    binaries: Vec<Product>,
+    fetcher: &dyn Fetcher,
+    install_path: PathBuf,
+    mut meta: PkgMeta,
+    opts: Options,
+    package: Package<Meta>,
+    pkg_path: PathBuf,
+    temp_dir: TempDir,
+) -> Result<(), anyhow::Error> {
     // Prompt user for third-party source
     if fetcher.is_third_party() {
         warn!(
@@ -279,3 +317,43 @@ async fn main() -> Result<(), anyhow::Error> {
     info!("Installation complete!");
     Ok(())
 }
+
+async fn install_from_source(opts: Options, package: Package<Meta>) -> Result<(), anyhow::Error> {
+    // Prompt user for source install
+    warn!("The package will be installed from source (with cargo)",);
+    if !opts.no_confirm && !opts.dry_run && !confirm()? {
+        warn!("Installation cancelled");
+        return Ok(());
+    }
+
+    if opts.dry_run {
+        info!(
+            "Dry-run: running `cargo install {} --version {} --target {}`",
+            package.name, package.version, opts.target
+        );
+        Ok(())
+    } else {
+        debug!(
+            "Running `cargo install {} --version {} --target {}`",
+            package.name, package.version, opts.target
+        );
+        let mut child = Command::new("cargo")
+            .arg("install")
+            .arg(package.name)
+            .arg("--version")
+            .arg(package.version)
+            .arg("--target")
+            .arg(opts.target)
+            .spawn()?;
+        debug!("Spawned command pid={:?}", child.id());
+
+        let status = child.wait().await?;
+        if status.success() {
+            info!("Cargo finished successfully");
+            Ok(())
+        } else {
+            error!("Cargo errored! {:?}", status);
+            Err(anyhow::anyhow!("Cargo install error"))
+        }
+    }
+}
